/****************************************************************************
** Meta object code from reading C++ file 'controladaptor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "controladaptor.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'controladaptor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14ControlAdaptorE_t {};
} // unnamed namespace

template <> constexpr inline auto ControlAdaptor::qt_create_metaobjectdata<qt_meta_tag_ZN14ControlAdaptorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ControlAdaptor",
        "D-Bus Interface",
        "org.kde.KMix.Control",
        "D-Bus Introspection",
        "  <interface name=\"org.kde.KMix.Control\">\n    <property access="
        "\"read\" type=\"s\" name=\"id\"/>\n    <property access=\"read\" t"
        "ype=\"s\" name=\"readableName\"/>\n    <property access=\"read\" t"
        "ype=\"s\" name=\"iconName\"/>\n    <property access=\"readwrite\" "
        "type=\"b\" name=\"mute\"/>\n    <property access=\"read\" type=\"b"
        "\" name=\"canMute\"/>\n    <property access=\"readwrite\" type=\"b"
        "\" name=\"recordSource\"/>\n    <property access=\"read\" type=\"b"
        "\" name=\"hasCaptureSwitch\"/>\n    <property access=\"readwrite\""
        " type=\"i\" name=\"volume\"/>\n    <property access=\"readwrite\" "
        "type=\"i\" name=\"absoluteVolume\"/>\n    <property access=\"read\""
        " type=\"i\" name=\"absoluteVolumeMin\"/>\n    <property access=\"r"
        "ead\" type=\"i\" name=\"absoluteVolumeMax\"/>\n    <method name=\""
        "increaseVolume\"/>\n    <method name=\"decreaseVolume\"/>\n    <me"
        "thod name=\"toggleMute\"/>\n  </interface>\n",
        "decreaseVolume",
        "",
        "increaseVolume",
        "toggleMute",
        "absoluteVolume",
        "absoluteVolumeMax",
        "absoluteVolumeMin",
        "canMute",
        "hasCaptureSwitch",
        "iconName",
        "id",
        "mute",
        "readableName",
        "recordSource",
        "volume"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'decreaseVolume'
        QtMocHelpers::SlotData<void()>(5, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'increaseVolume'
        QtMocHelpers::SlotData<void()>(7, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'toggleMute'
        QtMocHelpers::SlotData<void()>(8, 6, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'absoluteVolume'
        QtMocHelpers::PropertyData<int>(9, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'absoluteVolumeMax'
        QtMocHelpers::PropertyData<int>(10, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'absoluteVolumeMin'
        QtMocHelpers::PropertyData<int>(11, QMetaType::Int, QMC::DefaultPropertyFlags),
        // property 'canMute'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'hasCaptureSwitch'
        QtMocHelpers::PropertyData<bool>(13, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'iconName'
        QtMocHelpers::PropertyData<QString>(14, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'id'
        QtMocHelpers::PropertyData<QString>(15, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'mute'
        QtMocHelpers::PropertyData<bool>(16, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'readableName'
        QtMocHelpers::PropertyData<QString>(17, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'recordSource'
        QtMocHelpers::PropertyData<bool>(18, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'volume'
        QtMocHelpers::PropertyData<int>(19, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<ControlAdaptor, qt_meta_tag_ZN14ControlAdaptorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject ControlAdaptor::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractAdaptor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14ControlAdaptorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14ControlAdaptorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14ControlAdaptorE_t>.metaTypes,
    nullptr
} };

void ControlAdaptor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ControlAdaptor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->decreaseVolume(); break;
        case 1: _t->increaseVolume(); break;
        case 2: _t->toggleMute(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->absoluteVolume(); break;
        case 1: *reinterpret_cast<int*>(_v) = _t->absoluteVolumeMax(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->absoluteVolumeMin(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->canMute(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->hasCaptureSwitch(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->iconName(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->id(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->mute(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->readableName(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->recordSource(); break;
        case 10: *reinterpret_cast<int*>(_v) = _t->volume(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setAbsoluteVolume(*reinterpret_cast<int*>(_v)); break;
        case 7: _t->setMute(*reinterpret_cast<bool*>(_v)); break;
        case 9: _t->setRecordSource(*reinterpret_cast<bool*>(_v)); break;
        case 10: _t->setVolume(*reinterpret_cast<int*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *ControlAdaptor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ControlAdaptor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14ControlAdaptorE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int ControlAdaptor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    return _id;
}
QT_WARNING_POP
